<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://Techspawn.com
 * @since      1.0.0
 *
 * @package    Wamc
 * @subpackage Wamc/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Wamc
 * @subpackage Wamc/includes
 * @author     Techspawn <info@techspawn.com>
 */
class Wamc {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Wamc_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'WAMC_VERSION' ) ) {
			$this->version = WAMC_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'wamc';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Wamc_Loader. Orchestrates the hooks of the plugin.
	 * - Wamc_i18n. Defines internationalization functionality.
	 * - Wamc_Admin. Defines all hooks for the admin area.
	 * - Wamc_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wamc-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wamc-i18n.php';
		/**
		 * The class responsible for rest api functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wamc-rest-api.php';


		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wamc-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wamc-public.php';

		$this->loader = new Wamc_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Wamc_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Wamc_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Wamc_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );


		// menu init
		$this->loader->add_action('admin_menu', $plugin_admin, 'wamc_register_menu_page');
		$this->loader->add_action('admin_init', $plugin_admin, 'wamc_register_setting');

		// title change
		$this->loader->add_action('pre_get_document_title', $plugin_admin, 'wamc_change_404_title');
		
		//send bulk messages
		$this->loader->add_action('wp_ajax_wamc_send_bulk_messages', $plugin_admin, 'wamc_send_bulk_messages_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_send_bulk_messages', $plugin_admin, 'wamc_send_bulk_messages_callback');
		
		//save admin settings
		$this->loader->add_action('wp_ajax_wamc_save_admin_settings', $plugin_admin, 'wamc_save_admin_settings_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_save_admin_settings', $plugin_admin, 'wamc_save_admin_settings_callback');

		//save user settings
		$this->loader->add_action('wp_ajax_wamc_save_user_settings', $plugin_admin, 'wamc_save_user_settings_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_save_user_settings', $plugin_admin, 'wamc_save_user_settings_callback');


		//add new campaign
		$this->loader->add_action('wp_ajax_wamc_add_new_campaign', $plugin_admin, 'wamc_add_new_campaign_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_add_new_campaign', $plugin_admin, 'wamc_add_new_campaign_callback');
		
		//update userlist
		$this->loader->add_action('wp_ajax_wamc_update_userlist', $plugin_admin, 'wamc_update_userlist_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_update_userlist', $plugin_admin, 'wamc_update_userlist_callback');

		//update userlist
		$this->loader->add_action('wp_ajax_wamc_update_template_options', $plugin_admin, 'wamc_update_template_options_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_update_template_options', $plugin_admin, 'wamc_update_template_options_callback');

		//edit campaign
		$this->loader->add_action('wp_ajax_wamc_edit_campaign', $plugin_admin, 'wamc_edit_campaign_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_edit_campaign', $plugin_admin, 'wamc_edit_campaign_callback');

		//delete campaign
		$this->loader->add_action('wp_ajax_wamc_delete_campaign', $plugin_admin, 'wamc_delete_campaign_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_delete_campaign', $plugin_admin, 'wamc_delete_campaign_callback');

		//delete userlist
		$this->loader->add_action('wp_ajax_wamc_delete_userlist', $plugin_admin, 'wamc_delete_userlist_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_delete_userlist', $plugin_admin, 'wamc_delete_userlist_callback');

		//remove user from userlist
		$this->loader->add_action('wp_ajax_wamc_delete_user', $plugin_admin, 'wamc_delete_user_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_delete_user', $plugin_admin, 'wamc_delete_user_callback');


		//remove users from userlist
		$this->loader->add_action('wp_ajax_wamc_remove_user_from_ul', $plugin_admin, 'wamc_remove_user_from_ul_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_remove_user_from_ul', $plugin_admin, 'wamc_remove_user_from_ul_callback');


		//add new user list
		$this->loader->add_action('wp_ajax_wamc_add_new_userlist', $plugin_admin, 'wamc_add_new_userlist_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_add_new_userlist', $plugin_admin, 'wamc_add_new_userlist_callback');
		
		//add to current user list
		$this->loader->add_action('wp_ajax_wamc_addtocurr_new_userlist', $plugin_admin, 'wamc_addtocurr_new_userlist_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_addtocurr_new_userlist', $plugin_admin, 'wamc_addtocurr_new_userlist_callback');
		
		//add to current user list (by current users add)
		$this->loader->add_action('wp_ajax_wamc_add_to_current_userlist_user', $plugin_admin, 'wamc_add_to_current_userlist_user_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_add_to_current_userlist_user', $plugin_admin, 'wamc_add_to_current_userlist_user_callback');
		

		//add to current user list
		$this->loader->add_action('wp_ajax_wamc_update_userlist_otg', $plugin_admin, 'wamc_update_userlist_otg_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_update_userlist_otg', $plugin_admin, 'wamc_update_userlist_otg_callback');
		
		//add new user to userlist single
		$this->loader->add_action('wp_ajax_wamc_add_to_userlist_single', $plugin_admin, 'wamc_add_to_userlist_single_callback');
		$this->loader->add_action('wp_ajax_nopriv_wamc_add_to_userlist_single', $plugin_admin, 'wamc_add_to_userlist_single_callback');
		
		//load relational dropdown
		$this->loader->add_action('wp_ajax_wamc_all_package_table', $plugin_admin, 'wamc_all_package_table');
		$this->loader->add_action('wp_ajax_wamc_relatable_options', $plugin_admin, 'wamc_relatable_options');

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Wamc_Public( $this->get_plugin_name(), $this->get_version() );
		
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles', 999 );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Wamc_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
